@@ -1,8 +1,8 @@
 /***************************************************************************
                                 Aldo
                           --------------------
+  Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007 Giuseppe "denever" Martino
     begin                : Sun May 6 2001
-    copyright            : Giuseppe "denever" Martino
     email                : denever@users.sourceforge.net
  ***************************************************************************/
 /***************************************************************************
@@ -52,6 +52,7 @@ void OnTextFile()
     unsigned int dotlen = keyer_sec->option("DotLength")->get();
     unsigned int linelen = keyer_sec->option("LineLength")->get();
     unsigned int tone = keyer_sec->option("Tone")->get();
+    string       device = keyer_sec->option("Device")->getstr();
         
     string filename;
     do
@@ -62,7 +63,7 @@ void OnTextFile()
     }
     while(! getline(cin, filename) );
 
-    libaudiostream::AudioWorkSpace aws;
+    libaudiostream::AudioWorkSpace aws(device);
 
     libkeyer::Keyer current_keyer(aws, speed, charpause, strpause, dotlen, linelen);
 
